package hr.fer.grading;

import java.lang.reflect.Method;
import java.util.Locale;

import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.core.Filter;
import org.apache.logging.log4j.core.appender.ConsoleAppender;
import org.apache.logging.log4j.core.config.Configurator;
import org.apache.logging.log4j.core.config.builder.api.AppenderComponentBuilder;
import org.apache.logging.log4j.core.config.builder.api.ConfigurationBuilder;
import org.apache.logging.log4j.core.config.builder.api.ConfigurationBuilderFactory;
import org.apache.logging.log4j.core.config.builder.impl.BuiltConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class GradingRunner {
  private Logger log = LoggerFactory.getLogger(getClass());

  public static void main(String[] args) throws Exception {
    configureLogging();
    Locale.setDefault(Locale.US);
    new GradingRunner().run(args);
  }

  private void run(String[] args) throws Exception {
    log.debug("Arguments: {}", (Object)args);

    runGrading(args);

    // this is mandatory if we are testing GUI
    System.exit(0);
  }

  private void runGrading(String[] args) throws Exception {
    Class<?> junitRunnerClass = getClass().getClassLoader().loadClass("hr.fer.junit.grading.command_line.CommandLineRunner");
    Method mainMethod = junitRunnerClass.getMethod("main", String[].class);
    mainMethod.invoke(junitRunnerClass, (Object)args);
  }

  private static void configureLogging() {
    ConfigurationBuilder<BuiltConfiguration> builder = ConfigurationBuilderFactory.newConfigurationBuilder();
    builder.setStatusLevel(Level.ERROR);
    builder.setConfigurationName("BuilderTest");
    builder.add(builder.newFilter("ThresholdFilter", Filter.Result.ACCEPT, Filter.Result.NEUTRAL)
        .addAttribute("level", Level.DEBUG));
    AppenderComponentBuilder appenderBuilder = builder.newAppender("Stdout", "CONSOLE").addAttribute("target",
        ConsoleAppender.Target.SYSTEM_ERR);
    appenderBuilder.add(builder.newLayout("PatternLayout")
        .addAttribute("pattern", "%d [%t] %-5level: %msg%n%throwable"));
    appenderBuilder.add(builder.newFilter("MarkerFilter", Filter.Result.DENY, Filter.Result.NEUTRAL)
        .addAttribute("marker", "FLOW"));
    builder.add(appenderBuilder);
    builder.add(builder.newLogger("org.apache.logging.log4j", Level.DEBUG)
        .add(builder.newAppenderRef("Stdout")).addAttribute("additivity", false));
    builder.add(builder.newRootLogger(Level.ERROR).add(builder.newAppenderRef("Stdout")));
    Configurator.initialize(builder.build());
  }
}