package hr.fer.grading;

import java.io.PrintWriter;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class NetworkInterfacePrinter{

  private PrintWriter writer;
  private Stream<NetworkInterfaceWrapper> interfaces;

  public NetworkInterfacePrinter(PrintWriter writer) throws SocketException {
    this(writer, NetworkInterface.networkInterfaces().map(NetworkInterfaceWrapper::new));
  }

  public NetworkInterfacePrinter(PrintWriter writer, Stream<NetworkInterfaceWrapper> interfaces) {
    this.writer = writer;
    this.interfaces = interfaces;
  }

  public void print() throws SocketException {
    interfaces.forEach(this::print);
  }

  private void print(NetworkInterfaceWrapper ni) {
    writer.print("Name: ");
    writer.println(ni.getName());

    writer.print("DisplayName: ");
    writer.println(ni.getDisplayName());

    printInetAddresses(ni);
    printMacAddres(ni);
    printStatus(ni);
    printVirtual(ni);
    writer.println();
  }

  private void printVirtual(NetworkInterfaceWrapper ni) {
    writer.print("Virtual: ");
    writer.println(ni.isVirtual());
  }

  private void printStatus(NetworkInterfaceWrapper ni) {
    writer.print("Status: ");
    try {
      writer.println(ni.isUp() ? "active" : "inactive");
    } catch (SocketException e) {
      writer.println("unknown");
    }
  }

  private void printMacAddres(NetworkInterfaceWrapper ni) {
    writer.print("MAC: ");

    try {
      List<Integer> mac = ni.getMacAddress();
      if (mac != null) {
        writer.print(mac.stream()
          .map(macPart -> String.format("%02x", macPart))
          .collect(Collectors.joining(":")));
        writer.println();
      } else {
        writer.println("null");
      }
    } catch (SocketException e) {
      writer.println("unknown - exception: " + e.getMessage());
    }

  }

  private void printInetAddresses(NetworkInterfaceWrapper ni) {
    writer.println("InetAddresses:");
    ni.getInetAddresses()
      .forEach(addr -> {
        writer.print("  ");
        writer.println(addr.getHostAddress());
      });
  }

}
