package hr.fer.grading;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.when;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.List;
import java.util.stream.Stream;

import org.junit.jupiter.api.Test;
import org.mockito.Mockito;

class NetworkInterfacePrinterTest {

  @Test
  void printNetworkInterfaceNames() throws Exception {
    StringWriter outputWriter = new StringWriter();
    NetworkInterfacePrinter printer = new NetworkInterfacePrinter(new PrintWriter(outputWriter, true));

    printer.print();

    String output = outputWriter.toString();
    NetworkInterface.networkInterfaces()
      .forEach(ni -> {
        assertThat(output).contains(ni.getName());
        assertThat(output).contains(ni.getDisplayName());
      });
  }

  @Test
  void printInetAddresses() throws Exception {
    StringWriter outputWriter = new StringWriter();
    InetAddress address1 = InetAddress.getByAddress(new byte[] {(byte)161,53,19,28});
    InetAddress address2 = InetAddress.getByAddress(new byte[] {32,56,1,18});
    NetworkInterfaceWrapper ni = Mockito.mock(NetworkInterfaceWrapper.class);
    when(ni.getInetAddresses()).thenReturn(Stream.of(address1, address2));
    NetworkInterfacePrinter printer = new NetworkInterfacePrinter(
        new PrintWriter(outputWriter, true),
        Stream.of(ni));

    printer.print();

    String output = outputWriter.toString();
    assertThat(output).contains(
        " 161.53.19.28",
        " 32.56.1.18");
  }

  @Test
  void printMacAddresses() throws Exception {
    StringWriter outputWriter = new StringWriter();
    NetworkInterfaceWrapper ni = Mockito.mock(NetworkInterfaceWrapper.class);
    when(ni.getMacAddress()).thenReturn(List.of(Byte.toUnsignedInt((byte)0xff), 0xa0, 15, 0x5e));
    NetworkInterfacePrinter printer = new NetworkInterfacePrinter(
        new PrintWriter(outputWriter, true),
        Stream.of(ni));

    printer.print();

    String output = outputWriter.toString();
    assertThat(output).contains(
        "MAC: ff:a0:0f:5e");
  }

  @Test
  void printStatus() throws Exception {
    StringWriter outputWriter = new StringWriter();
    NetworkInterfaceWrapper ni = Mockito.mock(NetworkInterfaceWrapper.class);
    when(ni.isUp()).thenReturn(true);
    NetworkInterfacePrinter printer = new NetworkInterfacePrinter(
        new PrintWriter(outputWriter, true),
        Stream.of(ni));

    printer.print();

    String output = outputWriter.toString();
    assertThat(output).contains(
        "Status: active");
  }

  @Test
  void printVirtual() throws Exception {
    StringWriter outputWriter = new StringWriter();
    NetworkInterfaceWrapper ni = Mockito.mock(NetworkInterfaceWrapper.class);
    when(ni.isVirtual()).thenReturn(true);
    NetworkInterfacePrinter printer = new NetworkInterfacePrinter(
        new PrintWriter(outputWriter, true),
        Stream.of(ni));

    printer.print();

    String output = outputWriter.toString();
    assertThat(output).contains(
        "Virtual: true");
  }
}
